'use strict';

Object.defineProperty(exports, "__esModule", {
   value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides a file path resolver that is NPM module aware.
 *
 * @example
 * const pathResolver = new PathResolver('.', 'foo/bar.js', 'foo-bar', 'foo/bar.js');
 * pathResolver.importPath;            // 'foo-bar'
 * pathResolver.filePath;              // 'foo/bar.js'
 * pathResolver.resolve('./baz.js');   // 'foo/baz.js'
 */
var PathResolver = function () {
   /**
    * Instantiate PathResolver.
    *
    * @param {string} rootPath - root directory path.
    *
    * @param {string} filePath - relative file path from root directory path.
    *
    * @param {string} [packageName] - npm package name.
    *
    * @param {string} [mainFilePath] - npm main file path.
    */
   function PathResolver(rootPath, filePath) {
      var packageName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : void 0;
      var mainFilePath = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : void 0;
      (0, _classCallCheck3.default)(this, PathResolver);

      this.setPathData(rootPath, filePath, packageName, mainFilePath);
   }

   /**
    * Gets the absolute file path.
    *
    * @returns {string}
    */


   (0, _createClass3.default)(PathResolver, [{
      key: 'resolve',


      /**
       * Resolves the given file path with the data initialized in PathResolver.
       *
       * @param {string} relativePath - relative path on this file.
       *
       * @returns {string}
       */
      value: function resolve(relativePath) {
         if (typeof relativePath !== 'string') {
            throw new TypeError('\'relativePath\' is not a \'string\'.');
         }

         var selfDirPath = _path2.default.dirname(this._filePath);
         var resolvedPath = _path2.default.resolve(selfDirPath, relativePath);
         var resolvedRelativePath = _path2.default.relative(this._rootPath, resolvedPath);

         return this._slash(resolvedRelativePath);
      }

      /**
       * Resolve absolute path given the file path on this file.
       *
       * @param {string} relativePath - relative path on this file.
       *
       * @returns {string}
       */

   }, {
      key: 'resolveAbsolutePath',
      value: function resolveAbsolutePath(relativePath) {
         if (typeof relativePath !== 'string') {
            throw new TypeError('\'relativePath\' is not a \'string\'.');
         }

         var selfDirPath = _path2.default.dirname(this._filePath);
         var resolvedPath = _path2.default.resolve(selfDirPath, relativePath);

         return this._slash(resolvedPath);
      }

      /**
       * Sets the given path data to this PathResolver.
       *
       * @param {string} rootPath - root directory path.
       *
       * @param {string} filePath - relative file path from root directory path.
       *
       * @param {string} [packageName] - npm package name.
       *
       * @param {string} [mainFilePath] - npm main file path.
       */

   }, {
      key: 'setPathData',
      value: function setPathData(rootPath, filePath) {
         var packageName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : void 0;
         var mainFilePath = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : void 0;

         if (typeof rootPath !== 'string') {
            throw new TypeError('\'rootPath\' is not a \'string\'.');
         }
         if (typeof filePath !== 'string') {
            throw new TypeError('\'filePath\' is not a \'string\'.');
         }

         if (typeof packageName !== 'undefined' && typeof packageName !== 'string') {
            throw new TypeError('\'packageName\' is not a \'string\'.');
         }

         if (typeof mainFilePath !== 'undefined' && typeof mainFilePath !== 'string') {
            throw new TypeError('\'mainFilePath\' is not a \'string\'.');
         }

         /**
          * @type {string}
          */
         this._rootPath = _path2.default.resolve(rootPath);

         /**
          * @type {string}
          */
         this._filePath = _path2.default.resolve(filePath);

         /**
          * @type {NPMPackageObject}
          */
         this._packageName = packageName;

         if (mainFilePath) {
            /**
             * @type {string}
             */
            this._mainFilePath = _path2.default.resolve(mainFilePath);
         }
      }

      /**
       * Converts 'back slash' to 'slash' as necessary if the OS platform is windows.
       *
       * @param {string} filePath - Target file path.
       *
       * @returns {string} converted path.
       * @private
       */

   }, {
      key: '_slash',
      value: function _slash(filePath) {
         if (_os2.default.platform() === 'win32') {
            filePath = filePath.replace(/\\/g, '/');
         }

         return filePath;
      }
   }, {
      key: 'absolutePath',
      get: function get() {
         return this._slash(this._filePath);
      }

      /**
       * Gets the file path that is the relative path from the root dir.
       *
       * @returns {string}
       */

   }, {
      key: 'filePath',
      get: function get() {
         var relativeFilePath = _path2.default.relative(this._rootPath, this._filePath);

         return this._slash(relativeFilePath);
      }

      /**
       * Gets import path that is considered package name or main file and path prefix.
       *
       * @returns {string}
       */

   }, {
      key: 'importPath',
      get: function get() {
         var relativeFilePath = this.filePath;

         if (this._mainFilePath === _path2.default.resolve(relativeFilePath)) {
            return this._packageName;
         }

         var filePath = void 0;

         // If the relative file path starts with an actual relative path outside of where TJSDoc is executed then
         // use the relative file path as is...
         if (/^(?:\.\.)+/.test(relativeFilePath)) {
            filePath = relativeFilePath;
         } else {
            // If the package name is available then construct the relative import to the package / NPM module.
            if (this._packageName) {
               filePath = _path2.default.normalize('' + this._packageName + _path2.default.sep + relativeFilePath);
            } else {
               // Consider the relative path as local source.
               filePath = './' + relativeFilePath;
            }
         }

         return this._slash(filePath);
      }
   }]);
   return PathResolver;
}();

exports.default = PathResolver;
module.exports = exports['default'];